/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_ATTRIBUTE_H
#define DPC_IMPL_TYPE_ATTRIBUTE_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"
#include "impl_type_uint32.h"

namespace dpc {
struct attribute {
    ::Float curve{ 0.0F };
    ::Float offset{ 0.0F };
    ::UInt32 lane_marker_type{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(curve);
        fun(offset);
        fun(lane_marker_type);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(curve);
        fun(offset);
        fun(lane_marker_type);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("curve", curve);
        fun("offset", offset);
        fun("lane_marker_type", lane_marker_type);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("curve", curve);
        fun("offset", offset);
        fun("lane_marker_type", lane_marker_type);
    }

    friend bool operator==(const ::dpc::attribute& lhs, const ::dpc::attribute& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.curve - rhs.curve)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.offset - rhs.offset)) < DBL_EPSILON) && (lhs.lane_marker_type == rhs.lane_marker_type);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_ATTRIBUTE_H
